package com.perforce.hive;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hive.browse.BrowseControl;
import com.perforce.hive.common.JsonUtil;
import com.perforce.hive.connection.AccountControl;
import com.perforce.hive.connection.ConnectionConfig;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.hive.deploy.DeployControl;
import com.perforce.hive.depot.DepotControl;
import com.perforce.hive.depot.DepotDomain;
import com.perforce.hive.navigation.NavBar;
import com.perforce.hive.search.SearchControl;
import com.perforce.hive.view.ViewControl;

import freemarker.template.Configuration;
import spark.ModelAndView;
import spark.Spark;
import spark.template.freemarker.FreeMarkerEngine;

public class Main {

	private static Logger logger = LoggerFactory.getLogger(PluginArtifactFactory.class);

	public static void main(String[] args) throws Exception {

		String keyFile = ConnectionConfig.get().getKeystoreFile();
		String keyPass = ConnectionConfig.get().getKeystorePass();
		String trustFile = ConnectionConfig.get().getTrustFile();
		String trustPass = ConnectionConfig.get().getTrustPass();
		Spark.secure(keyFile, keyPass, trustFile, trustPass);

		Spark.port(ConnectionConfig.get().getPort());
		Spark.staticFileLocation("/public");

		PluginArtifactFactory.loadPlugins();
		start();
	}

	public static void start() throws Exception {

		Configuration configuration = PluginArtifactFactory.getConfiguration();

		Spark.before((request, response) -> {

			String path = request.pathInfo();

			if (logger.isDebugEnabled()) {
				String method = request.requestMethod();
				StringBuffer sb = new StringBuffer();
				for (String header : request.headers()) {
					String value = request.headers(header);
					sb.append(header + ":" + value + " ");
				}
				logger.debug(method + ":" + path + "(" + sb.toString() + ")");
			}

			// Connect Basic > Cookie > Session
			ConnectionSession.basicAuth(request, response);
			ConnectionSession.cookie(request, response);
			ConnectionSession.getAuth(request);

			DepotDomain reg = DepotDomain.getInstance();
			String map = reg.getMap(request);
			if (map != null) {
				int port = request.port();
				String protocol = ConnectionConfig.get().getProtocol();
				String host = ConnectionConfig.get().getHost();
				String redirect = protocol + "://" + host + ":" + port + map + path;

				// SimpleRouteMatcher routes = RouteMatcherFactory.get();
				// HttpMethod type = HttpMethod.valueOf(method.toLowerCase());
				// RouteMatch match = routes.findTargetForRequestedRoute(type,
				// map + path, "*/*");
				//
				// if (match == null) {
				// return;
				// }
				//
				// Object target = match.getTarget();
				// if (target == null) {
				// return;
				// }
				//
				// try {
				// if (target instanceof RouteImpl) {
				// RouteImpl route = ((RouteImpl) target);
				// Object body = route.handle(request, response);
				//
				//

				//
				// response.body((String) body);
				//
				// }
				// } catch (HaltException hEx) {
				// throw hEx;
				// }

				// for(String header : request.headers()) {
				// String value = request.headers(header);
				// response.header(header, value);
				// }

				// Enumeration<String> headers = request.raw().getHeaderNames();
				// for (String key : Collections.list(headers)) {
				// String value = request.raw().getHeader(key);
				// response.header(key, value);
				// }

				// String size = request.headers("Content-Length");

				// logger.info("redirect: " + redirect + " size:" + size);
				// response.redirect(redirect, 302);
			}
		});

		Spark.get("/", (request, response) -> {
			Map<String, Object> attributes;
			attributes = NavBar.attributes("cover", request);
			return new ModelAndView(attributes, "/page-cover.html");
		}, new FreeMarkerEngine(configuration));

		Spark.post("/login", (request, response) -> {
			return ConnectionSession.login(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/logout", (request, response) -> {
			ConnectionSession.logout(request, response);
			response.redirect("/");
			return null;
		});

		Spark.get("/account", (request, response) -> {
			return AccountControl.get(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/view/*", (request, response) -> {
			return ViewControl.view(request, response);
		});

		Spark.put("/view/:repo/*", (request, response) -> {
			return ViewControl.putDirect(request, response);
		});

		Spark.get("/search", (request, response) -> {
			return SearchControl.query(request, response);
		}, new FreeMarkerEngine(configuration));

		// BrowseControl

		Spark.get("/list", (request, response) -> {
			return BrowseControl.list(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/list/", (request, response) -> {
			return BrowseControl.list(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/browse", (request, response) -> {
			return BrowseControl.depots(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/browse/", (request, response) -> {
			return BrowseControl.depots(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/browse/*", (request, response) -> {
			return BrowseControl.browse(request, response);
		}, new FreeMarkerEngine(configuration));

		// DepotControl

		Spark.get("/config", (request, response) -> {
			return DepotControl.getListDepots(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/create/depot/:type/:site", (request, response) -> {
			return DepotControl.getCreateDepot(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.post("/create/depot", (request, response) -> {
			DepotControl.postCreateDepot(request, response);
			response.redirect("/config");
			return "";
		});

		Spark.get("/edit/depot/:repo", (request, response) -> {
			return DepotControl.getEditDepot(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.get("/protect/:depot", (request, response) -> {
			return DepotControl.getProtectDepot(request, response);
		}, JsonUtil.json());

		Spark.post("/obliterate/preview", (request, response) -> {
			return DepotControl.postObliteratePreview(request, response);
		}, JsonUtil.json());

		Spark.post("/obliterate/purge", (request, response) -> {
			return DepotControl.postObliteratePurge(request, response);
		}, JsonUtil.json());

		Spark.get("/info/:repo", (request, response) -> {
			return DepotControl.getDepotInfo(request, response);
		}, new FreeMarkerEngine(configuration));

		// DeployControl

		Spark.get("/submit/:type", (request, response) -> {
			return DeployControl.getSubmit(request, response);
		}, new FreeMarkerEngine(configuration));

		Spark.post("/submit/:type", (request, response) -> {
			return DeployControl.postSubmit(request, response);
		});

		Spark.get("/report/:type/*", (request, response) -> {
			return DeployControl.getReport(request, response);
		}, new FreeMarkerEngine(configuration));

		// Errors

		Spark.exception(Exception.class, (e, request, response) -> {
			response.status(500);
			response.body("Exception: " + e.getMessage());
		});

	}
}
