package com.perforce.hive.browse;

import com.perforce.p4java.core.file.IFileSize;
import com.perforce.p4java.core.file.IFileSpec;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DecimalFormat;

public class BrowseFile {

	private final String name;
	private final String link;
	private final String path;
	private final String date;
	private final long size;

	public BrowseFile(IFileSpec spec, IFileSize size) {
		String file = spec.getDepotPathString();
		this.path = file;

		file = file.replaceFirst("//", "");
		Path parts = Paths.get(file);
		this.name = parts.getFileName().toString();

		this.link = "/view/" + file;
		this.date = spec.getDate().toString();
		this.size = size.getFileSize();
	}

	public String getName() {
		return name;
	}

	public String getLink() {
		return link;
	}

	public String getPath() {
		return path;
	}

	public String getDate() {
		return date;
	}

	public String getSize() {
		if (size <= 0) {
			return "0";
		}

		String[] units = new String[] { "", "K", "M", "G", "T", "P" };
		int i = (int) (Math.log10(size) / Math.log10(1024));
		double n = size / Math.pow(1024, i);
		String fmt = new DecimalFormat("#,##0.#").format(n);
		return fmt + " " + units[i] + "Bytes";
	}

	public String toString() {
		return getName() + " (" + getLink() + ")";
	}
}
