package com.perforce.hive.deploy;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.perforce.hive.PluginArtifactFactory;
import com.perforce.hive.api.ChangeModel;
import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.common.AbstractControl;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.hive.navigation.NavBar;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import spark.ModelAndView;
import spark.Request;
import spark.Response;

public class DeployControl extends AbstractControl {

	public static ModelAndView getSubmit(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("deploy", request);
		model.put("type", getType(request));

		String pending = request.queryParams("pending");
		model.put("pending", pending);

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("files", DeployFactory.listFiles(p4, pending));
			p4.disconnect();
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/submit/page-submit.html");
	}

	public static String postSubmit(Request request, Response response) {
		try {
			// get type from request parameters
			ArtifactType type = getType(request);

			// get pending parameter
			String pendingStr = request.queryParams("pending");
			int pending = Integer.parseInt(pendingStr);

			// get description
			String desc = request.queryParams("descID");

			ArtifactInterface artifact = PluginArtifactFactory.getArtifact(type, request, response);
			ChangeModel change = artifact.submitPending(pending, desc);
			artifact.index();
			artifact.disconnect();

			response.redirect(
					"/report/" + type.getName() + "/" + change.getPath() + "?submitted=" + change.getChange());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}

	public static ModelAndView getReport(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("deploy", request);
		model.put("type", getType(request));

		String submitted = request.queryParams("submitted");
		model.put("submitted", submitted);

		String base = request.pathInfo();
		base = (base.endsWith("/")) ? base : base + "/";

		model.put("path", toBrowsePath(base));

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("files", DeployFactory.listFiles(p4, submitted));
			p4.disconnect();
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/submit/page-report.html");
	}

	private static String toBrowsePath(String base) {
		List<String> parts = new ArrayList<>();
		parts.addAll(Arrays.asList(base.split("/")));
		parts.remove(0);
		parts.remove(0);
		parts.remove(0);

		String path = String.join("/", parts);
		path = "/browse/" + path;
		return path;
	}
}