package com.perforce.hive.depot;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.google.gson.Gson;
import com.perforce.hive.PluginArtifactFactory;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.api.repository.RepoInterface;
import com.perforce.hive.api.repository.SiteType;
import com.perforce.hive.common.AbstractControl;
import com.perforce.hive.connection.ConnectionConfig;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.hive.navigation.ListItem;
import com.perforce.hive.navigation.NavBar;
import com.perforce.hive.site.GroupSite;
import com.perforce.p4java.admin.IProtectionEntry;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.core.file.IObliterateResult;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.impl.generic.admin.ProtectionEntry;
import com.perforce.p4java.option.server.ObliterateFilesOptions;
import com.perforce.p4java.server.IOptionsServer;

import spark.ModelAndView;
import spark.Request;
import spark.Response;

public class DepotControl extends AbstractControl {

	public static ModelAndView getCreateDepot(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("config", request);

		ArtifactType type = getType(request);
		model.put("type", type);
		model.put("site", getSite(request));

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("depots", DepotFactory.listByType(p4, type));
			p4.disconnect();
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/configuration/page-create.html");
	}

	public static void postCreateDepot(Request request, Response response) {
		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			RepoInterface repo = PluginArtifactFactory.create(request);
			DepotModel depot = new DepotModel(repo);
			depot.create(p4);

			String json = request.queryParams("acl");
			ProtectionEntry[] array = new Gson().fromJson(json, ProtectionEntry[].class);

			DepotProtect perms = new DepotProtect(p4);
			perms.setPermissions(depot.getName(), array);
			perms.save();

			p4.disconnect();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static ModelAndView getEditDepot(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("config", request);

		String repoID = getRepo(request);

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			DepotModel depot = DepotFactory.getByName(p4, repoID);
			model.put("depot", depot);

			RepoInterface repo = depot.getRepo();
			ArtifactType type = repo.getArtifactType();
			model.put("type", type);
			model.put("site", repo.getSiteType());

			if (repo instanceof GroupSite) {
				GroupSite group = (GroupSite) repo;
				model.put("depots", DepotFactory.listByType(p4, type));
				model.put("list", group.getList(p4));
			}

			p4.disconnect();
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/configuration/page-create.html");
	}

	public static ModelAndView getDepotInfo(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("browse", request);

		String repoID = getRepo(request);

		try {
			String port = String.valueOf(ConnectionConfig.get().getPort());
			model.put("port", port);

			InetAddress addr = InetAddress.getLocalHost();
			model.put("host", addr.getCanonicalHostName());

			IOptionsServer p4 = ConnectionSession.get(request, response);
			DepotModel depot = DepotFactory.getByName(p4, repoID);
			model.put("depot", depot);

			RepoInterface repo = depot.getRepo();
			ArtifactType type = repo.getArtifactType();
			model.put("type", type);

		} catch (P4JavaException e) {
			return error(e);
		} catch (UnknownHostException e) {
			return error(e);
		}

		return new ModelAndView(model, "/configuration/page-info.html");
	}

	public static ModelAndView getListDepots(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("config", request);

		// create list of depot types
		List<ListItem> types = new ArrayList<>();
		for (ArtifactType a : PluginArtifactFactory.getTypes()) {
			String title = a.getTitle();
			String path = "/create/depot/" + a.getName();
			types.add(new ListItem(title, path));
		}
		model.put("types", types);

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("stores", DepotFactory.listBySite(p4, SiteType.STORE));
			model.put("proxies", DepotFactory.listBySite(p4, SiteType.PROXY));
			model.put("groups", DepotFactory.listBySite(p4, SiteType.GROUP));
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/configuration/page-config.html");
	}

	public static List<IProtectionEntry> getProtectDepot(Request request, Response response) {
		String depot = request.params(":depot");
		List<IProtectionEntry> acl = new ArrayList<>();

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			DepotProtect perms = new DepotProtect(p4);
			acl = perms.getPermissions(depot);
			p4.disconnect();
		} catch (P4JavaException e) {
		}
		return acl;
	}

	public static ObliterateModel postObliteratePreview(Request request, Response response) {

		ObliterateModel model = new ObliterateModel();

		String path = request.queryParams("path");

		ObliterateFilesOptions opts = new ObliterateFilesOptions();
		opts.setExecuteObliterate(false);

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			List<IFileSpec> fileSpecs = FileSpecBuilder.makeFileSpecList(path);
			List<IObliterateResult> status = p4.obliterateFiles(fileSpecs, opts);
			for (IObliterateResult result : status) {
				model.addClient(result.getClientRecDeleted());
				model.addLabel(result.getLabelRecDeleted());
				model.addRev(result.getRevisionRecDeleted());
				model.addWorking(result.getWorkingRecDeleted());
				model.addList(result.getFileSpecs());
			}
			p4.disconnect();
		} catch (P4JavaException e) {
			return null;
		}
		return model;
	}

	public static ObliterateModel postObliteratePurge(Request request, Response response) {

		ObliterateFilesOptions opts = new ObliterateFilesOptions();
		opts.setExecuteObliterate(true);

		String base = request.queryParams("base");
		String path = request.queryParams("path");

		response.redirect(base);
		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			String username = p4.getUserName();
			String password = request.queryParams("password");
			ConnectionSession.login(username, password);
			p4.disconnect();
		} catch (P4JavaException e) {
			return null;
		}

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			List<IFileSpec> fileSpecs = FileSpecBuilder.makeFileSpecList(path);
			p4.obliterateFiles(fileSpecs, opts);
			p4.disconnect();
		} catch (P4JavaException e) {
			return null;
		}
		return null;
	}
}
