package com.perforce.hive.repository;

import java.lang.reflect.Type;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.perforce.hive.PluginArtifactFactory;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.api.repository.RepoInterface;
import com.perforce.hive.api.repository.SiteType;
import com.perforce.hive.site.GroupSite;
import com.perforce.hive.site.ProxySite;

public class AbstractRepoAdapter implements JsonDeserializer<RepoInterface> {

	@Override
	public RepoInterface deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
			throws JsonParseException {

		JsonObject jsonObject = json.getAsJsonObject();

		String typeStr = jsonObject.get("type").getAsString();
		ArtifactType type = PluginArtifactFactory.parseType(typeStr);

		String siteStr = jsonObject.get("site").getAsString();
		SiteType site = SiteType.parse(siteStr);

		if ("UNSUPPORTED".equals(type.getID()) || site == null) {
			return null;
		}

		Class<?> clazz = getClass(site, type);
		return context.deserialize(json, clazz);
	}

	private static Class<?> getClass(SiteType site, ArtifactType type) {

		switch (site) {
		case STORE:
			return PluginArtifactFactory.getRepositoryClass(type);

		case PROXY:
			return ProxySite.class;

		case GROUP:
			return GroupSite.class;

		default:
			return null;
		}
	}
}
