package com.perforce.hive.search;

import java.util.Map;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.perforce.hive.PluginArtifactFactory;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.common.AbstractControl;
import com.perforce.hive.connection.ConnectionConfig;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.depot.DepotModel;
import com.perforce.hive.navigation.NavBar;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import spark.ModelAndView;
import spark.Request;
import spark.Response;

public class SearchControl extends AbstractControl {

	private static Logger logger = LoggerFactory.getLogger(SearchControl.class);

	public static ModelAndView query(Request request, Response response) {

		Map<String, Object> model;
		model = NavBar.attributes("search", request);

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("repos", DepotFactory.listAll(p4));
			model.put("types", PluginArtifactFactory.getTypes());

			model.put(SearchID.TYPE.id(), SearchID.TYPE.get(request));
			model.put(SearchID.REPO.id(), SearchID.REPO.get(request));
			model.put(SearchID.FILE.id(), SearchID.FILE.get(request));
			model.put(SearchID.CONTENT.id(), SearchID.CONTENT.get(request));

			SearchQuery query = new SearchQuery(p4, request);

			// convert query to json string
			GsonBuilder gsonBuilder = new GsonBuilder();
			Gson gson = gsonBuilder.create();
			String jsonQuery = gson.toJson(query);
			logger.info(jsonQuery);

			// run search and add results to model
			ArtifactType type = getType(p4, request);
			SearchResults results = search(jsonQuery);
			model.put("results", results);

			ResultsModel resultsModel = new ResultsModel(results, type);
			model.put("table", resultsModel);

			p4.disconnect();

			return new ModelAndView(model, "page-search.html");
		} catch (Exception e) {
			e.printStackTrace();
			return error(e);
		}
	}

	private static SearchResults search(String jsonQuery) throws Exception {

		// POST request to p4 search
		HttpClient client = HttpClientBuilder.create().build();
		String p4search = ConnectionConfig.get().getSearch();
		HttpPost post = new HttpPost(p4search + "/api/search");
		post.addHeader("content-type", "application/json");
		StringEntity entity = new StringEntity(jsonQuery);
		post.setEntity(entity);
		HttpResponse r = client.execute(post);

		// Decode result
		String jsonResults = EntityUtils.toString(r.getEntity(), "UTF-8");
		logger.info(jsonResults);

		GsonBuilder gsonBuilder = new GsonBuilder();
		Gson gson = gsonBuilder.create();
		SearchResults results = gson.fromJson(jsonResults, SearchResults.class);
		logger.info(results.toString());
		return results;
	}

	private static ArtifactType getType(IOptionsServer p4, Request request) throws P4JavaException {
		// look to see if type was specified
		String typeStr = SearchID.TYPE.get(request);
		if (typeStr != null && !typeStr.isEmpty() && !"_any_".equals(typeStr)) {
			return PluginArtifactFactory.parseType(typeStr);
		}

		// look to see if a repo was specified
		String repoStr = SearchID.REPO.get(request);
		if (repoStr != null && !repoStr.isEmpty() && !"_all_".equals(repoStr)) {
			DepotModel depot = DepotFactory.getByName(p4, repoStr);
			return depot.getType();
		}

		return null;
	}

}
