package com.perforce.hive.plugin.gem;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.exec.CommandLine;
import org.apache.commons.exec.DefaultExecutor;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.FilenameUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.artifact.DefaultArtifact;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.depot.DepotModel;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import spark.Request;
import spark.Response;

public class GemArtifact extends DefaultArtifact implements ArtifactInterface {

	private static Logger logger = LoggerFactory.getLogger(GemArtifact.class);

	private String gemID;
	private String version;
	private List<FileItem> fileItems = new ArrayList<>();

	public GemArtifact(Request request, Response response) throws Exception {
		super(request, response);
	}

	public GemArtifact(IOptionsServer p4, String repoID) throws Exception {
		super(p4);
		this.repoID = repoID;
	}

	@Override
	public void processForm(Request request) throws FileUploadException {
		DiskFileItemFactory factory = new DiskFileItemFactory();
		ServletFileUpload up = new ServletFileUpload(factory);
		List<FileItem> items = up.parseRequest(request.raw());

		for (FileItem item : items) {
			if (item.isFormField()) {
				String fieldname = item.getFieldName();
				String fieldvalue = item.getString();

				switch (fieldname) {
				case "repoID":
					repoID = fieldvalue;
					break;
				case "gemID":
					gemID = fieldvalue;
					break;
				case "version":
					version = fieldvalue;
					break;
				}
			} else {
				fileItems.add(item);
			}
		}

		path = Paths.get(repoID, "gems");
	}

	@Override
	public DepotModel getDepot() throws P4JavaException {
		DepotModel depot = DepotFactory.getByName(p4, repoID);
		return depot;
	}

	@Override
	public void index() throws Exception {
		versionIndex();
	}

	@Override
	public void buildIndex() throws Exception {
		logger.info("running: gem generate_index");

		CommandLine cmdLine = new CommandLine("gem");
		cmdLine.addArgument("generate_index");

		File root = new File("index", repoID);

		DefaultExecutor executor = new DefaultExecutor();
		executor.setWorkingDirectory(root);
		executor.execute(cmdLine);
	}
}
