package com.perforce.hive.plugin.vagrant;

import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.artifact.DefaultArtifact;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.depot.DepotModel;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import spark.Request;
import spark.Response;

import java.io.File;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

public class VagrantArtifact extends DefaultArtifact implements ArtifactInterface {

	private static Logger logger = LoggerFactory.getLogger(VagrantArtifact.class);

	private String provider = "";
	private String name = "";
	private String version = "";

	public VagrantArtifact(Request request, Response response) throws Exception {
		super(request, response);
	}

	public VagrantArtifact(IOptionsServer p4, String repoID) throws Exception {
		super(p4);
		this.repoID = repoID;
	}

	@Override
	public void processForm(Request request) throws FileUploadException {
		DiskFileItemFactory factory = new DiskFileItemFactory();
		ServletFileUpload up = new ServletFileUpload(factory);
		List<FileItem> items = up.parseRequest(request.raw());

		for (FileItem item : items) {
			if (item.isFormField()) {
				String fieldname = item.getFieldName();
				String fieldvalue = item.getString();

				switch (fieldname) {
				case "repoID":
					repoID = fieldvalue;
					break;
				case "provider":
					provider = fieldvalue;
					break;
				case "name":
					name = fieldvalue;
					break;
				case "version":
					version = fieldvalue;
					break;
				}
			} else {
				fileItems.add(item);
			}
		}

		// set path
		path = Paths.get(repoID);

		// set filename
		filename = name + "-" + provider + "-" + version + ".box";
	}

	@Override
	public List<String> populateMultiPartPost() throws Exception {

		List<String> list = new ArrayList<>();

		File root = new File(getClient().getRoot());

		if(fileItems.size() != 1) {
			throw new P4JavaException("Only one Vagrant box can be uploaded.");
		}
		
		for (FileItem item : fileItems) {
			File file = new File(root, filename);
			item.write(file);
			list.add(file.getPath());
			logger.info("uploading: " + filename);
		}
		return list;
	}
	
	@Override
	public DepotModel getDepot() throws P4JavaException {
		DepotModel depot = DepotFactory.getByName(p4, repoID);
		return depot;
	}

	@Override
	public void index() throws Exception {
		// no indexing
	}

	@Override
	public void buildIndex() throws Exception {
		// no indexing
	}
}
