package com.perforce.hive.plugin.vagrant;

import java.io.InputStream;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import com.perforce.hive.api.DefaultEngine;
import com.perforce.hive.api.Loader;
import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.api.artifact.ResultInterface;
import com.perforce.hive.api.artifact.SummaryInterface;
import com.perforce.hive.api.repository.RepoInterface;
import com.perforce.hive.api.search.ResultDetailed;
import com.perforce.p4java.core.IDepot;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.server.IOptionsServer;

import ro.fortsoft.pf4j.Extension;
import ro.fortsoft.pf4j.Plugin;
import ro.fortsoft.pf4j.PluginWrapper;
import spark.Request;
import spark.Response;
import spark.Spark;

public class VagrantPlugin extends Plugin {

	private static String name;
	private static ArtifactType type;

	public VagrantPlugin(PluginWrapper wrapper) {
		super(wrapper);
		name = wrapper.getPluginId();
		type = new ArtifactType(name, "VAGRANT");
		type.setTitle("Vagrant Artifact");
	}

	public static ArtifactType getType() {
		return type;
	}

	@Extension
	public static class VagrantLoader implements Loader {

		@Override
		public ArtifactType getType() {
			return type;
		}

		@Override
		public void loadImage(String endpoint) {
			Spark.get(endpoint, (request, response) -> {
				InputStream image = VagrantPlugin.class.getResourceAsStream("/images/vagrant.png");
				HttpServletResponse raw = DefaultEngine.rawImage(image, response);
				image.close();
				return raw;
			});
		}

		@Override
		public ArtifactInterface getArtifact(Request request, Response response) throws Exception {
			return new VagrantArtifact(request, response);
		}

		@Override
		public ArtifactInterface getProxy(IOptionsServer p4, String repoID) throws Exception {
			return new VagrantArtifact(p4, repoID);
		}

		@Override
		public SummaryInterface getSummary(IOptionsServer p4, List<IFileSpec> summary) {
			return new VagrantSummary(p4, summary);
		}

		@Override
		public ResultInterface getResult(ResultDetailed result) {
			return new VagrantResult(result);
		}

		@Override
		public RepoInterface createRepository(ArtifactType type) {
			return new VagrantRepository(type);
		}

		@Override
		public RepoInterface createRepository(Request request) {
			return new VagrantRepository(request);
		}

		@Override
		public RepoInterface createRepository(IDepot depot) {
			// Not implemented
			return null;
		}

		@Override
		public Class<?> getRepositoryClass() {
			return VagrantRepository.class;
		}
	}

}
